from setuptools import setup, find_packages

setup(
    author='Chris Pickett',
    author_email='chris.pickett@gmail.com',
    description='A light-weight blogging app for Django.',
    license='MIT',
    long_description=open('README.rst').read(),
    name='django-hermes',
    packages=find_packages(),
    url='https://github.com/bunchesofdonald/django-hermes',
    version='1.3.0',
)
